/* src/add_info.c - Ajout d'informations en mmoire
 * Copyright (C) 2002-2004 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * $Id: add_info.c,v 1.68 2004/04/14 13:52:47 romexzf Exp $
 */

#include "main.h"
#include "debug.h"
#include "outils.h"
#include "hash.h"
#include "ban.h"
#include "cs_cmds.h"

#ifdef USE_MEMOSERV
void add_memo(anUser *nick, const char *de, time_t date, const char *message, int read)
{
	aMemo *new = calloc(1, sizeof *new), *tmp = nick->memohead;

	if(!new)
	{
		Debug(W_MAX, "add_memo, malloc a chou pour aMemo %s (%s)", nick->nick, message);
		return;
	}

	new->next = NULL;
	Strncpy(new->de, de, NICKLEN);
	new->date = date;
	new->read = read;
	Strncpy(new->message, message, MEMOLEN);

	if(!tmp) nick->memohead = new;
	else
	{
		for(;tmp->next;tmp = tmp->next);
		tmp->next = new;
	}
}
#endif

struct badchans *add_abadchan(const char *chaine, const char *raison)
{
	struct badchans *ptr = malloc(sizeof *ptr);

	if(!ptr)
	{
		Debug(W_MAX, "add_abadchan, malloc a chou pour le badchan %s", chaine);
		return NULL;
	}

	Strncpy(ptr->chaine, chaine, 99);
	Strncpy(ptr->raison, raison, 149);
	ptr->next = badchead;
	badchead = ptr;
	return ptr;
}

anAccess *add_access(anUser *user, const char *chan, int level, int flag, time_t lastseen)
{
	anAccess *ptr = calloc(1, sizeof *ptr), *tmp;
	aChan *c;
	aLink *lp;

	if(!ptr)
	{
		Debug(W_MAX, "add_access, malloc a chou pour l'accs %s (%s)", user->nick, chan);
		return NULL;
	}

	ptr->next = NULL;
	ptr->level = level;
	ptr->flag = flag;
	ptr->user = user;
	ptr->info = NULL;
	ptr->lastseen = lastseen ? lastseen : user->lastseen;

	if((c = getchaninfo(chan))) ptr->c = c;
	else
	{
		Debug(W_DESYNCH, "add_access, accs de %s sur %s, non reg ?", user->nick, chan);
		free(ptr);
		return NULL;
	}

	if(!user->accesshead) user->accesshead = ptr;
	else
	{
		for(tmp = user->accesshead;tmp->next && tmp->c != c;tmp = tmp->next);
		if(!tmp->next) tmp->next = ptr;
		else
		{
			Debug(W_DESYNCH, "add_access, accs de %s sur %s dj prsent?", user->nick, chan);
			free(ptr);
			return NULL;
		}
	}

	lp = malloc(sizeof *lp);
	lp->value.a = ptr;
	lp->next = c->access;
	c->access = lp;
	if(ptr->level == OWNERLEVEL) c->owner = ptr;
	return ptr;
}

void add_killinfo(aNick *nick, enum TimerType type)
{
	aKill *new = malloc(sizeof *new), *tmp = killhead;

	if(!new)
	{
		Debug(W_MAX|W_WARN, "add_killinfo, malloc a chou pour aKill %s", nick->nick);
		return;
	}

	new->next = NULL;
	new->nick = nick;
	Strncpy(new->against, nick->nick, NICKLEN);
	new->timestamp = CurrentTS;
	new->type = type;
	if(!tmp) killhead = new;
	else
	{
		for(;tmp->next;tmp = tmp->next);
		tmp->next = new;
	}
}

int add_abadnick(const char *chaine, int flag)
{
	struct badnicks *ptr = malloc(sizeof *ptr);

	if(!ptr)
		return Debug(W_MAX|W_WARN, "add_abadnick, malloc a chou pour le badnick %s", chaine);

	Strncpy(ptr->chaine, chaine, NICKLEN);
	ptr->flag = flag;
	ptr->next = badnhead;
	badnhead = ptr;
	return 1;
}

void add_join(aNick *nick, const char *chan, int status, time_t timestamp, aChan *c)
{
	aJoin *new = calloc(1, sizeof *new), *join;
	anAccess *a = NULL;
	aLink *lp;
	aBan *ban = NULL;

	if(!new)
	{
		Debug(W_MAX|W_WARN, "add_join, malloc a chou pour aJoin de %s sur %s", nick->nick, chan);
		return;
	}

	new->next = NULL;
	Strncpy(new->channel, chan, CHANLEN);
	new->status = (status & ~J_BURST);
	new->timestamp = timestamp;
	new->nick = nick;
	new->chan = NULL;
	if(!nick->joinhead) nick->joinhead = new;
	else
	{
		for(join = nick->joinhead;join->next && strcasecmp(join->channel, chan);join = join->next);
		if(!join->next && strcasecmp(join->channel, chan)) join->next = new;
		else /* le mec est dja sur le chan ? */
		{
			free(new);
			Debug(W_DESYNCH|W_WARN, "reJOIN %s de %s", nick->nick, chan);
			new = join;
		}
	}

	if((!(status & J_BURST) && !(c = getchaninfo(chan))) || !c || IsSuspend(c)) return;

	new->chan = c;
	c->users++;

	/* on ajoute le link vers chanmembers uniquement si le salon n'est PAS suspendu */
	lp = malloc(sizeof *lp);
	lp->value.j = new;
	lp->next = c->members; /* insertion du link dans les chan members*/
	c->members = lp;

	if(!CJoined(c)) /* Channel was empty [left, not joined] */
	{
		SetCJoined(c); /* rejoin it, restore default config */
		putserv("%s "TOKEN_JOIN" %s", cs.num, chan);
		DeOp(new);
		c->chanstamp = timestamp;

		if(status & J_BURST) /* B token outside my burst, need to join/op and clear modes (if any) */
			putserv("%s CM %s ovpsmikblrcCO", cs.num, chan);

		if(!(c->defmodes.modes & (C_MKEY|C_MLIMIT))) /* no parameter, can op myself */
			putserv("%s "TOKEN_MODE" %s +%so %s", cs.num, chan, GetCModes(c->defmodes), cs.num);
		else
		{ /* argh! need two lines (note that I should be in if there's a key..) */
			putserv("%s "TOKEN_MODE" %s +%s", cs.num, chan, GetCModes(c->defmodes));
			putserv("%s "TOKEN_MODE" %s +o %s", cs.num, chan, cs.num);
		}
		memcpy(&c->mode, &c->defmodes, sizeof c->mode);
		if(*c->deftopic) cstopic(c, c->deftopic);
	}

	/* check bans if not a burst join */
	if(!(status & J_BURST) && !(nick->flag & (N_OPER | N_SERVICE))
		&& !IsAnAdmin(nick->user) && (ban = is_ban(nick, c)))
	{
			csmode(c, MODE_OBV, "+b %s", ban->mask);
			cskick(chan, nick->numeric, "Enforce: %s", ban->raison);
			return; /* just got banned, nothing more to do ! */
	}

	/* Ok, he is not banned, perform join */
	a = (nick->user && (a = GetAccessIbyUserI(nick->user, c)) && !ASuspend(a)) ? a : NULL;
	/* he has an access, mark him as on channel (burst or not) */
	if(a) a->lastseen = 1;

	if(status & J_BURST) return; /* it's a Burst, nothing more to do.. */

	if(CSetWelcome(c) && *c->welcome) /* send welcome if needed */
		csntc(nick, GetReply(nick, L_WELCOMEJOIN), c->chan, c->welcome);

	if(!a) /* he has no access, and it's outside a burst */
	{	/* if he had created this channel because I wasn't in, he doesn't deserve its +o */
		if(status & J_OP) csmode(c, MODE_OBV, "-o %s", nick->numeric);
		return; /* remove it and return */
	}

	/* Perform priviledged(access-ed) user checks */
	if(AVoice(a)) DoVoice(new); /* auto op/voice */
	if(!CNoOps(c) && AOp(a)) DoOp(new);

	if(new->status) csmode(c, MODE_OBV, "+%s%s %s %s", IsOp(new) ? "o" : "", IsVoice(new) ? "v" : "",
			IsOp(new) ? nick->numeric : "" , IsVoice(new) ? nick->numeric : "");
	else if(status & J_OP) csmode(c, MODE_OBV, "-o %s", nick->numeric);

	if(a->info) /* finally, send infoline */
		putserv("%s "TOKEN_PRIVMSG" %s :[\2%s\2] %s", cs.num, chan, nick->user->nick, a->info);

	return;
}

void add_ban(aChan *chan, const char *mask, const char *raison, const char *de,
	time_t debut, time_t fin, int level)
{
	aBan *new = calloc(1, sizeof *new), *banp = chan->banhead;

	if(!new)
	{
		Debug(W_MAX|W_WARN, "add_ban, malloc a chou pour aBan %s sur %s de %s (%s)",
			mask, chan->chan, de, raison);
		return;
	}

	Strncpy(new->mask, mask, 99);
	str_dup(&new->raison, raison);
	Strncpy(new->de, de, NICKLEN);
	new->debut = debut;
	new->fin = fin ? debut + fin : 0;
	new->level = level;

	if(!banp || (fin && (!banp->fin || banp->fin >= new->fin)))
	{
		new->next = banp;
		if(banp) banp->last = new;
		chan->banhead = new;
		new->last = NULL;
	}
	else
	{
		for(;banp->next && (!fin || (banp->next->fin && new->fin >= banp->next->fin));banp = banp->next);

		new->last = banp;
		new->next = banp->next;
		if(banp->next) banp->next->last = new;
		banp->next = new;
	}
}

void add_ignore(aNick *nick, time_t expiration)
{
	struct ignore *new = malloc(sizeof *new);

	if(!new)
	{
		Debug(W_MAX|W_WARN, "add_ignore, malloc a chou pour l'ignore de %s", nick->host);
		return;
	}

	strcpy(new->host, nick->base64);
	new->exp = expiration;

	new->next = ignorehead;
	ignorehead = new;
}

